/* $Id: Personalization.js,v 1.1.2.2 2005/06/16 09:23:31 shanmugampl Exp $ */
/**
 *
 */
var _methodContainer = null;

/**
 * Contains the models (javascript objects) of all the views that have been
 * selected/added to list 
 */
var viewObjects = new Object();

/**
 *
 */
var originalViewObjects = new Object(); 

/**
 * Contains the list of view names that have been selected for layouting.
 */
var originalViewNames = new Array();

/**
 * Contains the list of view names that have been added to the layout. This
 * will always be a subset of originalViewNames
 */
var viewNames = new Array();

/**
 * Contains the number of columns for which the matrix is to be created.
 */
var _columnCount = 0;

/**
 * Contains the number of rows for which the matrix is to be created.
 */
var rowCount = 0;

/**
 * The core of this TableLayout component is this tableMatrix. Contians the
 * occupational details of various views. 
 */
var tableMatrix = new Array();

/**
 * Lists the values that are required for the construction of model (javascript
 * objects) for a given view. 
 */
var _viewPropertyNames = new Array("VIEWID", "TITLE", "DESCRIPTION", "ROWINDEX", "COLUMNINDEX", "COLSPAN", "ROWSPAN");

/**
 *
 */
var isStructureChanged = true;

/**
 * The methodContainer should be  named like, a name followed by _MC as 'PSLayout_MC'. 
 */
function initMethodContainer(personalizedView){
	if(parent[personalizedView] != null){
		_methodContainer = parent[personalizedView];
	}
}

/**
 *
 */
function executeElementCall(methodName, dataObj){
  if(_methodContainer != null && _methodContainer[methodName] != null){
    return _methodContainer[methodName](dataObj);
  }
  else
  {
    return parent[methodName](dataObj);
  }
}

/**
 * Used to create the default/original objects.
 */
function addViewObject(viewProperties){
	var count = _viewPropertyNames.length;
	var rowObject = new Object();
	var duplicateRow = new Object();
	for(var i = 0; i < count; i++){
		duplicateRow[_viewPropertyNames[i]] = viewProperties[i];
		rowObject[_viewPropertyNames[i]] = viewProperties[i];
	}
	viewObjects[viewProperties[0]] = rowObject;
	originalViewObjects[viewProperties[0]] = duplicateRow;
}

/**
 * Used to create an object for a dummy column / inserted column during runtime.
 */
function addRuntimeObject(viewProperties){
	var count = _viewPropertyNames.length;
	var rowObject = new Object();
	for(var i = 0; i < count; i++){
		rowObject[_viewPropertyNames[i]] = viewProperties[i];
	}
	viewObjects[viewProperties[0]] = rowObject;
}

/**
 * This is the initializer method for the table layout component. When this
 * method is called, it is assumed that the model (javascript objects) for the
 * views that are to be layed out are constructed already. This creates the
 * table matrix for the specified size and fills in the values accordingly.
 * Once the table matrix is completely filled, the model is reconstructed and
 * the table is created.
 *
 * @param	views		a list of view names that are to be layed out.
 * @param	columns		the number of columns required for the table matrix
 * @param	rows		the number of rows required for the table matrix.
 */
function init(views, columns, rows){
	originalViewNames = views;
	_columnCount = columns;
	rowCount = rows;
	isStructureChanged = false;
	
	// Construct the table matrix;
	if(views.length == 0){
		_columnCount = 1; 
		rowCount = 1;
		isRowColAdded = true;
	}

	tableMatrix = new Array(rowCount);
	for(var i = 0; i < rowCount; i++){
		tableMatrix[i] = new Array(_columnCount);
	}
	isStructureChanged = true;
	initTableMatrix();
	reCalculateTableData();
	executeElementCall("createTable");
}

/**
 * Populates the table matrix with the view names at appropriate places. 
 */
function initTableMatrix(){
	var size = originalViewNames.length;
	for(var i = 0; i < size; i++){
		var viewProps = viewObjects[originalViewNames[i]];
		var	viewName = viewProps["VIEWID"];
		var rowSpan = viewProps["ROWSPAN"];
		var colspan = viewProps["COLSPAN"];
		var rowIndex = viewProps["ROWINDEX"];
		var columnIndex = viewProps["COLUMNINDEX"];
		// For dummy views and views that are added newly, the rowIndex will be
		// -1. In such cases skip the current iteration and continue. 
		//
		// TODO : 
		//
		// 	i) Search for an empty place in the current row for placing the view.
		//  ii)If not available search for the next empty space. Should be
		//  quite intelligent in placing the values.
		if(rowIndex < 0){
			continue;
		}
		// Place the viewname for the given column and row index.
		tableMatrix[rowIndex][columnIndex] = viewName;
		// If the row span is greater than 1, then the other cells should also
		// be filled. In a 3*3 matrix for a first view with rowspan 2, the
		// result will be as follows
		// 	1	D	D
		//	1	D	D
		//	D	D	D
		//
		if(rowSpan > 1){
			for(var rows = 1; rows < rowSpan; rows++){
				tableMatrix[rowIndex + rows][columnIndex] = viewName;
			}
		}
		// If the column span is greater than 1, then the other cells should
		// also be filled. In a 3*3 matrix for a first view with colspan 2, the
		// result will be as follows
		// 	1	1	D
		//	D	D	D
		//	D	D	D
		//
		// If the column span and rowpsan are greater than 1, then in a 3*3
		// matrix for a first view with colspan 2 and rowspan 2, the result
		// will be as follows
		// 	1	1	D
		//	1	1	D
		//	D	D	D
		//
		// The second example is handled in the inner loop.

		if(colspan > 1){
			for(var cols = 1; cols < colspan; cols++){
				tableMatrix[rowIndex][columnIndex + cols] = viewName;
				for(var rows = 1; rows < rowSpan; rows++){
					tableMatrix[rowIndex + rows][columnIndex + cols] = viewName;
				}
			}
		}
	}
}

function checkAndAddEndPlaceHolders(){
	if(rowCount > 0 && _columnCount > 0){
		for(var rows = 0; rows < rowCount; rows ++){
			for(var cols = 0; cols < _columnCount; cols++){
				var iteratedViewName = tableMatrix[rows][cols];
				if(iteratedViewName.indexOf("Dummy")>= 0){
					tableMatrix[rows][cols] = "Dummy" + rows + cols;
				}
			}
		}
		removeEmptyRows();
		removeEmptyColumns();

		//checkPlaceHolderAtRow(true);
		//checkPlaceHolderAtRow(false);
		//checkPlaceHolderAtColumn(true);
		//checkPlaceHolderAtColumn(false);
		addPlaceHolders();
updateDummyValues();
	}
}

function addPlaceHolders(){
	var isPresent = true;
	for(var i = 0; i < _columnCount;i++){
		var viewName = tableMatrix[0][i];
		if(viewName.indexOf("DummyEnd") < 0){
			isPresent = false;
		}
	}
	if(!isPresent){
		tableMatrix.splice(0,0,new Array(_columnCount));
		for(var count = 0; count < _columnCount; count++){
			tableMatrix[0][count] = "DummyEnd" + 0 + count;
		}
		rowCount++;
	}
	isPresent = true;
	for(var i = 0; i < _columnCount;i++){
		var viewName = tableMatrix[rowCount-1][i];
		if(viewName.indexOf("DummyEnd") < 0){
			isPresent = false;
		}
	}
	if(!isPresent){
		tableMatrix[rowCount] = new Array(_columnCount);
		for(var count = 0; count < _columnCount; count++){
			tableMatrix[rowCount][count] = "DummyEnd" + rowCount + count;
		}
		rowCount++;
	}

	isPresent = true;
	for(var i = 0; i < rowCount; i++){
		var viewName = tableMatrix[i][0];
		if(viewName.indexOf("DummyEnd") < 0){
			isPresent = false;
		}
	}
	if(!isPresent){
		for(var j=0;j<rowCount;j++){
			tableMatrix[j].splice(0,0,"DummyEnd"+j+"0");
		}
		_columnCount++;
	}
	isPresent = true;
	for(var i = 0; i < rowCount; i++){
		var viewName = tableMatrix[i][_columnCount - 1];
		if(viewName.indexOf("DummyEnd") < 0){
			isPresent = false;
		}
	}
	if(!isPresent){
		for(var count = 0; count < rowCount; count++){
			tableMatrix[count][_columnCount] = "DummyEnd" + count + _columnCount;		
		}
		_columnCount++;
	}
}

function checkPlaceHolderAtColumn(isFirstColumn){
	var columnIndex = 0;
	if(!isFirstColumn){
		columnIndex = _columnCount - 1;
	}
	var isPresent = true;
	for(var i = 0; i < rowCount; i++){
		var viewName = tableMatrix[i][columnIndex];
		if(viewName.indexOf("DummyEnd") < 0){
			isPresent = false;
		}
	}
	if(!isPresent){
		for(var i = 0; i < rowCount; i++){
			var viewName = tableMatrix[i][columnIndex];
			if(viewName.indexOf("DummyEnd") >= 0){
				tableMatrix[i][columnIndex] = "Dummy" + i + columnIndex;
			}
		}
	}
}

function checkPlaceHolderAtRow(isFirstRow){
	var rowIndex = 0;
	if(!isFirstRow){
		rowIndex = rowCount - 1;
	}
	var isPresent = true;
	for(var i = 0; i < _columnCount;i++){
		var viewName = tableMatrix[rowIndex][i];
		if(viewName.indexOf("DummyEnd") < 0){
			isPresent = false;
		}
	}
	if(!isPresent){
		for(var i = 0; i < _columnCount;i++){
			var viewName = tableMatrix[rowIndex][i];
			if(viewName.indexOf("DummyEnd") >= 0){
				tableMatrix[rowIndex][i] = "Dummy" + rowIndex + i;
			}
		}
	}
}

/**
 * Updates the models of all the views that are added to the table matrix.
 * Takes care of setting the values like rowIndex, columnIndex, colspan etc..so
 * that these can be used during table construction.
 */
function reCalculateTableData(){
	reset();
	checkTableMatrix();
	checkAndAddEndPlaceHolders();
	if(!isRowColAdded){
		removeEmptyRows();
		removeEmptyColumns();
	}
	else{
		isRowColAdded = false;
	}
	var size = tableMatrix.length;
	var tempViews = new Array();
	for(var rows = 0; rows < size; rows ++) {
		var columns = tableMatrix[rows];
		for(var cols = 0; cols < columns.length; cols++){
			// Fetch the viewname in a cell of the tablematrix. If the viewname
			// is Dummy[rows][cols] add a Dummy object to the list of models 
			var viewName = tableMatrix[rows][cols];
			if(viewName == "Dummy"+rows+cols){
				executeElementCall("addRuntimeObject", new Array(viewName, "Dummy", null, rows, cols, 1, 1,rows,cols));		
				var viewProps = viewObjects[viewName];
				viewProps["ISDUMMY"] = true;
			}
			else if(viewName == "DummyEnd"+rows+cols){
				executeElementCall("addRuntimeObject", new Array(viewName, "DummyEnd", null, rows, cols, 1, 1,rows,cols));		
				var viewProps = viewObjects[viewName];
				viewProps["ISDUMMY"] = true;
			}
			else{
				var viewProps = viewObjects[viewName];
				var x = viewProps["X"];
				var y = viewProps["Y"];
				// Initially the x and y values will be -1. During each
				// occurence the value will be set to the corresponding cols
				// and rows.
				// A view might span across rows amd cols. During each
				// occurence, depending on the change in the x and y values the
				// colspan and rowspan values will be added.
				// For a column with colspan 2 and rowspan 2, the values will
				// be filled as 
				// 	cols	rows	x	y	colspan	rowspan
				//	0	0	-1	-1	1	1
				//	1	0	0	0	2	1
				//  	0	1	1	0	2	2
				// 	1	1	1	1	2	2
				//	
				if(cols != y){
					var colspan = viewProps["COLSPAN"];
					if(viewProps["ROWINDEX"] == -1 || viewProps["ROWINDEX"] == rows){
						viewProps["COLSPAN"] = colspan + 1;
					}
				}
				if(rows != x){
					var rowspan = viewProps["ROWSPAN"];
					viewProps["ROWSPAN"] = rowspan + 1;
				}
			}
			var present = false;
			// Create a new list with the views that have added to the table
			// matrix alone and assign it to viewNames array.
			for(var count = 0; count < tempViews.length; count ++){
				if(tempViews[count] == viewName){
					present = true;
					break;
				}
			}
			if(!present){
				tempViews[tempViews.length] = viewName;
			}
			// Set the staring position for a view.
			if(viewProps["ROWINDEX"] == -1){
				viewProps["ROWINDEX"] = rows;
				viewProps["COLUMNINDEX"] = cols;
			}
			viewProps["Y"] = cols;
			viewProps["X"] = rows;
		}
	}
	viewNames = tempViews;
	executeElementCall("disableSelectedViews");
}

/**
 * Reset the values of rowspan, columnspan, rowindex, columnindex, x, y values
 * of all view models.
 */
function reset(){
	var size = originalViewNames.length;
	if(rowCount <= 0 || _columnCount <= 0){
		rowCount = 1;
		_columnCount = 1;
		tableMatrix = new Array(rowCount);
		for(var i = 0; i < rowCount; i++){
			tableMatrix[i] = new Array(_columnCount);
		}
	}
	if(viewNames.length > 0){
	 size = viewNames.length;
	}
	for(var count = 0; count < size; count++){
		var viewProps = null;;
		if(viewNames.length > 0){
			viewProps = viewObjects[viewNames[count]];
		}
		else {
			viewProps = viewObjects[originalViewNames[count]];
		}
		viewProps["ROWSPAN"] = 0;
		viewProps["COLSPAN"] = 0;
		viewProps["ROWINDEX"] = -1;
		viewProps["COLUMNINDEX"] = -1;
		viewProps["X"] = -1;
		viewProps["Y"] = -1;
	}
}

/**
 * Performs a serious of checks on the tablematrix and corrects any mistakes.
 */
function checkTableMatrix(){
	removeDuplicateRows();
	fillEmptyCells();
	removeScatteredViews();
}

/**
 * There might be some situations wherein the same set of values will be
 * repeated in 2 consecutive rows.Such a situation will be turned to as follows
 * 	1	2	3	4	--->	1	2	3	4
 *	1	5	5	6			1	5	5	6
 *	1	5	5	6			7	8	8	9
 *	7	8	8	9
 * Also this step cannot be skipped because when all the values in two rows are equal, then the browser will interpret the next td as a continuation of the first one as
 * 	1	2		1	2
 *	3	4		3	4
 *	3	4		3	4	5	6
 *	5	6
 */
function removeDuplicateRows(){
	for(rows = 0; rows < tableMatrix.length; rows ++) {
		columns = tableMatrix[rows];
		if(rows + 1 < tableMatrix.length){
			columns1 = tableMatrix[rows + 1];
			if(columns.toString() == columns1.toString()){
				tableMatrix.splice(rows, 1);
				rows--;
				rowCount--;
			}
		}
	}
}

/**
 * In a matrix due to variations in rowindex and columnindex of views, some
 * cells might not be filled. These  will be replaced with Dummy values
 */
function fillEmptyCells(){
	for(var rows = 0; rows < rowCount; rows ++){
		var columns = tableMatrix[rows];
		for(cols = 0; cols < _columnCount; cols++){
			iteratedViewName = tableMatrix[rows][cols];
			if(iteratedViewName == null || iteratedViewName == _columnCount){
				tableMatrix[rows][cols] = "Dummy" + rows + cols;
			}
		}
	}
}

/**
 * Due to some expansion of width/height, the views might not be in sequence.
 * For example, consider a view like
 * 	1	2	3	4
 *	5	6	6	6
 *	7	8	8	9
 * If the user clicks on 3 to increase its rowspan, then the matrix will be 
 *	1	2	3	4
 *	5	6	3	6
 *	7	8	8	9
 * Here you can see that the view 6 is split which is logically wrong. This
 * case will be turned to one as shown by the following method.
 *	1	2	3	4
 *	5	6	3	D
 *	7	8	8	9
 */
function removeScatteredViews(){
	for(var viewCount = 0; viewCount < originalViewNames.length; viewCount++){
		var minX = minY = maxX = maxY = 0;
		var viewName = originalViewNames[viewCount];
		var viewProps = viewObjects[viewName];
		var isFirst = false;
		// Determine the min and max values for a given view. In our case it is
		// 1,1 and 1,3.
		for(var rows = 0; rows < tableMatrix.length; rows ++){
			var columns = tableMatrix[rows];
			for(var cols = 0; cols < columns.length; cols++){
				var iteratedViewName = tableMatrix[rows][cols];
				if(iteratedViewName == viewName){
					if(!isFirst){
						minX = rows;
						minY = cols;
					}
					isFirst = true;
					maxX = rows;
					maxY = cols;
				}
			}
		}
		// Now iterate for these values. When there is a break, then, for the
		// following iterations  set the cell value to Dummy.
		var isBreak = false;
		for(var rows = minX; rows <= maxX; rows++){
			for(var cols = minY; cols <= maxY; cols++){
				var value = tableMatrix[rows][cols];
				if(isBreak){
					tableMatrix[rows][cols] = "Dummy" + rows + cols;
				}
				if(value != viewName){
					isBreak = true;
				}
			}
		}
	}
}

var currentViewInEdit = null;

function enableEdit(index){
	currentViewInEdit = viewNames[index];
	isStructureChanged = false;
	createTable();
}

function disableEdit(){
	currentViewInEdit = null;
	isStructureChanged = false;
	createTable();
}

function constructRemovedViews(){
	var size = removedViews.length;
	if(size == 0){
		return;
	}
	var count = 0;
	var content = "";
	var html = document.getElementById('RemovedGridView').innerHTML;
	for(var i = size-1; i >= 0; i--){
		var isPresent = false;
		for(var j = 0; j < viewNames.length; j++){
			if(viewNames[j] == removedViews[i]){
				isPresent = true;
				break;
			}
		}
		if(!isPresent && count < 5){
			var obj = viewObjects[removedViews[i]]
			var newHtml = html.replace(/\$ID/g, i);
			newHtml = newHtml.replace('$VN', obj["VIEWID"]);
			newHtml = newHtml.replace('$DESC', obj["DESCRIPTION"]);
			newHtml = newHtml.replace(/\$TITLE/g, obj["TITLE"]);
			content = content.concat(newHtml);
			count++
		}
	}
	if(count > 0){
		document.getElementById('_RmTab').innerHTML = "<TD><br>" + content + "</TD>";
	}
	else {
		document.getElementById('_RmTab').innerHTML = "<TD align='center'><br>No recently removed components<br><br></TD>";
	}
}
var removedViews = new Array();
/**
 *
 */
function removeView(index, reload){
	for(var i=0; i<removedViews.length;i++){
		if(removedViews[i] == viewNames[index]){
			removedViews.splice(i,1);
		}
	}
	removedViews[removedViews.length] = viewNames[index];	
	if(reload){
		var viewName = viewNames[index];
		replaceCellsWithDummyValue(viewName);
		reCalculateTableData();
		createTable();
	}
}

/**
 *
 */
function replaceCellsWithDummyValue(viewName){
	for(rows = 0; rows < rowCount; rows ++){
		columns = tableMatrix[rows];
		for(cols = 0; cols < _columnCount; cols++){
			iteratedViewName = tableMatrix[rows][cols];
			if(iteratedViewName == viewName){
				tableMatrix[rows][cols] = "Dummy" + rows + cols;
			}
		}
	}
}

/**
 *
 */
function replaceCellsWithUpdatedValue(oldViewName, viewName){
	for(rows = 0; rows < rowCount; rows ++){
		columns = tableMatrix[rows];
		for(cols = 0; cols < _columnCount; cols++){
			iteratedViewName = tableMatrix[rows][cols];
			if(iteratedViewName == oldViewName){
				tableMatrix[rows][cols] = viewName;
			}
		}
	}
}

function updateCellValues(colIndex){
	if(colIndex == _columnCount - 1){
		return;
	}
	for(var count=0; count < rowCount; count++){
		if(tableMatrix[count][colIndex-1] == tableMatrix[count][colIndex+1]){
			tableMatrix[count][colIndex] = tableMatrix[count][colIndex+1]
		}
	}
}

function updateRowValues(rowIndex){
	if(rowIndex == rowCount - 1){
		return;
	}
	for(var count=0; count < _columnCount; count++){
		if(tableMatrix[rowIndex-1][count] == tableMatrix[rowIndex+1][count]){
			tableMatrix[rowIndex][count] = tableMatrix[rowIndex-1][count]
		}
	}
}

function isInvasionPossibleHorizontally(x, y, direction){
	var viewName = currentViewInEdit;
	viewProps = viewObjects[viewName];
	rowspan = viewProps["ROWSPAN"];
	isPossible = true;
	if(direction>0 && y==0){
		return false;
	}
	if(direction == 0){
		if(y == _columnCount-1){
			return false;
		}
		for(i=0; i<rowspan;i++){
			if(tableMatrix[x - i][y + 1].indexOf("Dummy") < 0){
				isPossible = false;
			}
		}
	}
	else {
		for(i=0; i<rowspan;i++){
			if(tableMatrix[x - i][y - 1].indexOf("Dummy") < 0){
				isPossible = false;
			}
		}
	}
	return isPossible;
}

function isInvasionPossibleVertically(x, y, direction){
	var viewName = currentViewInEdit;
	viewProps = viewObjects[viewName];
	colspan = viewProps["COLSPAN"];
	isPossible = true;
	if(direction>0 && x==0){
		return false;
	}
	if(direction == 0){
		if(x == rowCount-1){
			return false;
		}
		for(i=0; i<colspan;i++){
			if(tableMatrix[x + 1][y - i].indexOf("Dummy") < 0){
				isPossible = false;
			}
		}
	}
	else {
		for(i=0; i<colspan;i++){
			if(tableMatrix[x - 1][y - i].indexOf("Dummy") < 0){
				isPossible = false;
			}
		}
	}
	return isPossible;
}

/**
 *
 */
function increaseWidthRight(x, y){
	var viewName = currentViewInEdit;
	viewProps = viewObjects[viewName];
	rowspan = viewProps["ROWSPAN"];
	if(!isInvasionPossibleHorizontally(x,y,0)){
		for(var i=0; i<rowCount;i++){
			tableMatrix[i].splice(y+1,0,"Dummy"+i+(y+1));
		}
		_columnCount++;
		updateCellValues(y+1);
		updateDummyValues();
	}
	if(rowspan > 1){
		//	If the clicked cell is 7
		//	1	2	3		1	2	3
		//	4	4	5		4	4	5
		//	6	7	5		6	7	7
		//	8	7	D		8	7	7
		for(i=0; i<rowspan;i++){
			tableMatrix[x - i][y + 1] = viewName;
		}
	}
	else{
		tableMatrix[x][y + 1] = viewName;	
	}
	reCalculateTableData();
	createTable();
}

function increaseWidthLeft(x, y){
	var viewName = currentViewInEdit;
	viewProps = viewObjects[viewName];
	rowspan = viewProps["ROWSPAN"];
	colspan = viewProps["COLSPAN"];
	if(colspan > 1){
		y = y - colspan + 1;
	}
	if(!isInvasionPossibleHorizontally(x,y,1)){
		for(var i=0; i<rowCount;i++){
			tableMatrix[i].splice(y,0,"Dummy"+i+(y));
		}
		_columnCount++;
		updateCellValues(y);
		updateDummyValues();
		y++;
	}

	if(rowspan > 1){
		//	If the clicked cell is 7
		//	1	2	3		1	2	3
		//	4	4	5		4	4	5
		//	6	7	5		6	7	7
		//	8	7	D		8	7	7
		for(i=0; i<rowspan;i++){
			tableMatrix[x - i][y-1] = viewName;
		}
	}
	else{
		tableMatrix[x][y-1] = viewName;	
	}
	reCalculateTableData();
	createTable();
}

/**
 *
 */
function increaseHeightBottom(x, y){
	var viewName = currentViewInEdit;
	viewProps = viewObjects[viewName];
	colspan = viewProps["COLSPAN"];
	if(!isInvasionPossibleVertically(x,y,0)){
		tableMatrix.splice(x+1,0,new Array(_columnCount));
		for(count = 0; count < _columnCount; count++){
			tableMatrix[x+1][count] = "Dummy" + (x +1)+ count;
		}
		rowCount++;
		updateRowValues(x+1);
		updateDummyValues();
	}
	if(colspan > 1){
		//	If the clicked cell is 4
		//	1	2	3		1	2	3
		//	4	4	5		4	4	5
		//	6	7	8		4	4	8
		//	9	9	9		9	9	9
		for(i=0; i<colspan;i++){
			tableMatrix[x + 1][y - i] = viewName;
		}
	}
	else{
		tableMatrix[x + 1][y] = viewName;	
	}
	reCalculateTableData();
	createTable();
}

function increaseHeightTop(x, y){
	var viewName = currentViewInEdit;
	viewProps = viewObjects[viewName];
	colspan = viewProps["COLSPAN"];
	rowSpan = viewProps["ROWSPAN"];
	if(rowSpan > 1){
		x = x - rowSpan + 1;
	}
	if(!isInvasionPossibleVertically(x,y,1)){
		tableMatrix.splice(x,0,new Array(_columnCount));
		for(count = 0; count < _columnCount; count++){
			tableMatrix[x][count] = "Dummy" + x + count;
		}
		rowCount++;
		updateRowValues(x);
		updateDummyValues();
		x++;
	}

	if(colspan > 1){
		//	If the clicked cell is 4
		//	1	2	3		1	2	3
		//	4	4	5		4	4	5
		//	6	7	8		4	4	8
		//	9	9	9		9	9	9
		for(i=0; i<colspan;i++){
			tableMatrix[x - 1][y - i] = viewName;
		}
	}
	else{
		tableMatrix[x - 1][y] = viewName;	
	}
	reCalculateTableData();
	createTable();
}

/**
 *
 */
function decreaseWidthLeft(x, y){
	var viewName = currentViewInEdit;
	viewProps = viewObjects[viewName];
	rowspan = viewProps["ROWSPAN"];
	if(rowspan > 1){
		for(i=0; i<rowspan;i++){
			tableMatrix[x-i][y] = "Dummy"+(x-i)+(y);
		}
	}
	else{
		tableMatrix[x][y] = "Dummy"+x+y;
	}
	reCalculateTableData();
	createTable();
}

function decreaseWidthRight(x, y){
	var viewName = currentViewInEdit;
	viewProps = viewObjects[viewName];
	rowspan = viewProps["ROWSPAN"];
	colspan = viewProps["COLSPAN"];
	if(colspan > 1){
		y = y - (colspan - 1);
	}

	if(rowspan > 1){
		for(i=0; i<rowspan;i++){
			tableMatrix[x-i][y] = "Dummy"+(x-i)+y;
		}
	}
	else{
		tableMatrix[x][y] = "Dummy"+x+y;
	}
	reCalculateTableData();
	createTable();
}

/**
 *
 */
function decreaseHeightBottom(x, y){
	var viewName = currentViewInEdit;
	viewProps = viewObjects[viewName];
	colspan = viewProps["COLSPAN"];
	if(colspan > 1){
		for(i=0; i<colspan;i++){
			tableMatrix[x][y-i] = "Dummy"+x+(y-i);
		}
	}
	else{
		tableMatrix[x][y] = "Dummy"+x+y;
	}
	reCalculateTableData();
	createTable();
}

function decreaseHeightTop(x, y){
	var viewName = currentViewInEdit;
	viewProps = viewObjects[viewName];
	colspan = viewProps["COLSPAN"];
	rowspan = viewProps["ROWSPAN"];
	if(rowspan > 1){
		x = x - (rowspan - 1);
	}

	if(colspan > 1){
		for(i=0; i<colspan;i++){
			tableMatrix[x][y-i] = "Dummy"+(x)+(y-i);
		}
	}
	else{
		tableMatrix[x][y] = "Dummy"+(x)+y;
	}
	reCalculateTableData();
	createTable();
}

/**
 *
 */
function addEmptyRow(){
	tableMatrix[rowCount] = new Array(_columnCount);
	for(count = 0; count < _columnCount; count++){
		tableMatrix[rowCount][count] = "Dummy" + rowCount + count;
	}
	rowCount++;
}

/**
 *
 */
function addRow(){
	isRowColAdded = true;
	addEmptyRow();
	reCalculateTableData();
	createTable();
}

/**
 *
 */
function deleteRow(){
	for(var count = 0; count < _columnCount; count++) {
		var viewName = tableMatrix[rowCount - 1][count];
		for(var i = 0; i < viewNames.length;i++){
			if(viewNames[i] == viewName){
				removeView(i, false);
			}
		}
	}
	tableMatrix.splice(rowCount - 1, 1);
	rowCount--;
	reCalculateTableData();
	createTable();
}

var isRowColAdded = false;

/**
 *
 */
function addColumn(){
	isRowColAdded = true;
	for(count = 0; count < rowCount; count++){
		tableMatrix[count][_columnCount] = "Dummy" + count + _columnCount;
	}
	_columnCount++;
	reCalculateTableData();
	createTable();
}

/**
 *
 */
function deleteColumn(){
	for(var count = 0; count < rowCount; count++) {
		var viewName = tableMatrix[count][_columnCount - 1];
		for(var i = 0; i < viewNames.length;i++){
			if(viewNames[i] == viewName){
				removeView(i, false);
			}
		}
	}

	for(count = 0; count < rowCount; count++){
		rowArray = tableMatrix[count];
		rowArray.splice(_columnCount - 1, 1);
	}
	_columnCount--;
	reCalculateTableData();
	createTable();
}


var draggedViewName = null;

function insertView(draggedViewObj, insertId){
	currentViewInEdit = null;
	draggedViewName  = draggedViewObj.getAttribute("viewName");
	var draggedViewTitle  = draggedViewObj.getAttribute("title");
	var draggedViewDesc  = draggedViewObj.getAttribute("desc");

	var htmlObject = executeElementCall("addRuntimeObject", new Array(draggedViewName, draggedViewTitle, draggedViewDesc, -1, -1, 1, 1));
	var insertBefore = document.getElementById(insertId).getAttribute('viewId');
	viewNames[viewNames.length] = draggedViewName;
	if(insertId.indexOf('_ev') >= 0){
		if(insertBefore >= 0){
			var insObj = viewObjects[viewNames[insertBefore]];
			var insRowIndex = insObj["ROWINDEX"];
			if(insObj["VIEWID"].indexOf("Dummy") >= 0){
				tableMatrix[insObj["X"]][insObj["Y"]] = draggedViewName;
			}
			else {
				tableMatrix.splice(insRowIndex, 0, new Array(_columnCount));
				for(var i = 0; i < _columnCount; i++){
					tableMatrix[insRowIndex][i] = "Dummy" + insRowIndex + i;
				}	
				rowCount++;
				updateRowValues(insRowIndex);
				tableMatrix[insRowIndex][insObj["COLUMNINDEX"]] = draggedViewName;
				updateDummyValues();
			}
		}
		else {
			tableMatrix.splice(rowCount, 0, new Array(_columnCount));
			for(var i = 0; i < _columnCount; i++){
				tableMatrix[rowCount][i] = "Dummy" + insRowIndex + i;
			}
			tableMatrix[rowCount][0] = draggedViewName;
			rowCount++;
		}
		reCalculateTableData();
		createTable();
	}
	return true;	
}

function changeView(htmlObject, insertId){
	currentViewInEdit = null;
	var insertBefore = document.getElementById(insertId).getAttribute('viewId');
	var currentId = htmlObject.getAttribute('viewId');
	draggedViewName  = htmlObject.getAttribute("viewName");
	var insObj = viewObjects[viewNames[insertBefore]];
	var currObj = viewObjects[viewNames[currentId]];
	
	if(insertId.indexOf('_ev') >= 0){
		replaceCellsWithDummyValue(currObj["VIEWID"]);
		if(insertBefore >= 0){
			if(insObj["VIEWID"].indexOf("Dummy") >= 0){
				tableMatrix[insObj["X"]][insObj["Y"]] = currObj["VIEWID"];
			}
			else {
				moveView(currObj, insObj);
			}
		}
		else {
				tableMatrix.splice(rowCount, 0, new Array(_columnCount));
				for(var i = 0; i < _columnCount; i++){
					tableMatrix[rowCount][i] = "Dummy" + rowCount + i;
				}
				tableMatrix[rowCount][0] = draggedViewName;
				rowCount++;
		}
		updateDummyValues();
		reCalculateTableData();
		createTable();
	}
}

function removeEmptyRows(){
	for(var i = 0; i < tableMatrix.length; i++){
		if(i > 0 && i < tableMatrix.length -1){
			var isPresent = false;
			for(var j = 0; j < tableMatrix[0].length; j++) {
				if(tableMatrix[i][j].indexOf("Dummy") < 0){
					isPresent = true;
				}
			}
			if(!isPresent){
				tableMatrix.splice(i, 1);
				rowCount--;
			}
		}
	}
	if(tableMatrix.length == 0){
		rowCount = 1;
		_columnCount = 1;
		tableMatrix = new Array(rowCount);
		for(var i = 0; i < rowCount; i++){
			tableMatrix[i] = new Array(_columnCount);
		}
		fillEmptyCells();
	}
	updateDummyValues();
} 

function removeEmptyColumns(){
	for(var i = 0; i < tableMatrix[0].length; i++){
		if(i > 0 && i < tableMatrix[0].length -1){
			var isPresent = false;
			for(var j = 0; j < tableMatrix.length; j++) {
				if(tableMatrix[j][i].indexOf("Dummy") < 0){
					isPresent = true;
				}
			}
			if(!isPresent){
				for(var j = 0; j < tableMatrix.length; j++) {
					tableMatrix[j].splice(i, 1);
				}
				_columnCount--;
			}
		}
	}
	if(tableMatrix[0].length == 0){
		rowCount = 1;
		_columnCount = 1;
		tableMatrix = new Array(rowCount);
		for(var i = 0; i < rowCount; i++){
			tableMatrix[i] = new Array(_columnCount);
		}
		fillEmptyCells();
	}
	updateDummyValues();
} 
function moveView(currObj, insObj){
	var insRowIndex = insObj["ROWINDEX"];
	var insColIndex = insObj["COLUMNINDEX"];
	var rowSpan = currObj["ROWSPAN"];
	var columnSpan = currObj["COLSPAN"];
	for(var i = 0; i < rowSpan; i++){
		tableMatrix.splice(insRowIndex, 0, new Array(_columnCount));
		rowCount++;
	}
	for(var j = insRowIndex; j < insRowIndex + rowSpan; j++){
		for(var i = 0; i < _columnCount; i++){
			tableMatrix[j][i] = "Dummy" + j + i;
		}	
	}
	if(insColIndex + columnSpan > _columnCount){
		for(var j = _columnCount; j < insColIndex + columnSpan; j++){
			for(var i = 0; i < rowCount; i++){
				tableMatrix[i][j] = "Dummy" + i + j;
			}	
			_columnCount++;
		}
	}
	for(var i = 0; i < rowSpan; i++){
		for(var j = 0; j < columnSpan; j++){
			tableMatrix[i + insRowIndex][j + insColIndex] = currObj["VIEWID"];
		}
	}
}

function enableViewSelection(object){
	showDescription(object);
	if(draging){
		var id = object.id;
		var index = object.getAttribute("viewId");
		disableViewSelection();

		showDescription(object);
		enabledView = id;
		if(id.indexOf("_evl") >= 0 || id.indexOf("_dv") >= 0){
			if(object.className == 'emptyCell'){
				object.className = 'emptyCellHover';
			}
			else {
				object.className = 'emptyCellEndHover';
			}
		}
		else if(id.indexOf("_ev") >= 0){
			document.getElementById("_indicator1" + index).className = 'show';
//			document.getElementById("_indicator2" + index).className = 'show';
//			document.getElementById("_indicator3" + index).className = 'show';
		}
	}
}

/**
 *
 */
function disableViewSelection(){
	if(enabledView != null){
		var preIndex = document.getElementById(enabledView).getAttribute("viewId");
		if(enabledView.indexOf("_evl") >= 0 || enabledView.indexOf("_dv") >= 0){
			if(document.getElementById(enabledView).className == 'emptyCellHover'){
				document.getElementById(enabledView).className = 'emptyCell';
			}
			else {
				document.getElementById(enabledView).className = 'emptyCellEnd';
			}
		}
		else {
			document.getElementById("_indicator1" + preIndex).className = 'hide';
//			document.getElementById("_indicator2" + preIndex).className = 'hide';
//			document.getElementById("_indicator3" + preIndex).className = 'hide';
		}
		hideDescription();
	}
}

function updateDummyValues(){
	for(rows = 0; rows < rowCount; rows ++){
		columns = tableMatrix[rows];
		for(cols = 0; cols < _columnCount; cols++){
			iteratedViewName = tableMatrix[rows][cols];
			if(iteratedViewName.indexOf("DummyEnd")>= 0){
				tableMatrix[rows][cols] = "DummyEnd" + rows + cols;
			}
			else if(iteratedViewName.indexOf("Dummy") >= 0){
				tableMatrix[rows][cols] = "Dummy" + rows + cols;
			}
		}
	}
}

function disableSelectedViews(doc){
	if(doc == null || doc == 'undefined'){
		doc = document;
	}
	if(doc != null){
		var obj = doc.getElementsByTagName("table");
		for(var i = 0; i < obj.length; i++){
			var id = obj[i].id;
			if(id.indexOf("LV") < 0){
				continue;
			}
			var viewName = obj[i].getAttribute("viewName");
			var present = false;
			for(var j = 0; j < viewNames.length; j++){
				if(viewName == viewNames[j]){
					present = true;
					obj[i].className = 'viewSelected';
				}
			}
			if(!present){
				obj[i].className = 'viewNotSelected cursor';
			}
		}
	}
}

function showDescription(obj){
	var id = obj.id;
	var desc = obj.getAttribute('desc');
	if(desc != 'null'){
		document.getElementById('_ViewDesc').innerHTML = desc;
	}
	else {
		document.getElementById('_ViewDesc').innerHTML = 'No description available for <i>' + obj.getAttribute('title') + '</i>';
	}
}

function hideDescription(){
	document.getElementById('_ViewDesc').innerHTML = "";
}


var undoRedoIndex = -1;
var isFromUndoRedo = false;
var undoRedoArray = new Array();

function undo(){
	undoRedoIndex--;
	var tempMatrix = undoRedoArray[undoRedoIndex];
	tableMatrix = getDuplicateMatrix(tempMatrix);	
	_columnCount = tableMatrix[0].length;
	rowCount = tableMatrix.length;
	isFromUndoRedo = true;
	reCalculateTableData();
	createTable();	
}
function redo(){
	undoRedoIndex++;
	var tempMatrix = undoRedoArray[undoRedoIndex];
	tableMatrix = getDuplicateMatrix(tempMatrix);	
	_columnCount = tableMatrix[0].length;
	rowCount = tableMatrix.length;
	isFromUndoRedo = true;
	reCalculateTableData();
	createTable();
}

function hidePane(){
	document.getElementById("_HIDEPANE").className = 'paneBg hide';
	document.getElementById("_SHOWPANE").className = 'paneBg show';
	document.getElementById("_PANE").className = 'rightBg hide';
}

function showPane(){
	document.getElementById("_HIDEPANE").className = 'paneBg show';
	document.getElementById("_SHOWPANE").className = 'paneBg hide';
	document.getElementById("_PANE").className = 'rightBg show';
}

function getDuplicateMatrix(originalMatrix){
	var duplicateMatrix = new Array(originalMatrix.length);
	for(var rc = 0; rc < originalMatrix.length; rc++){
		duplicateMatrix[rc] = new Array(originalMatrix[0].length);
		for(var cc = 0; cc < originalMatrix[0].length; cc++){
			duplicateMatrix[rc][cc] = originalMatrix[rc][cc];  
		}
	}
	return duplicateMatrix;
}

/**
 *
 */
function createTable(){
	var width = (_columnCount*250) + "px";
	if(!isFromUndoRedo && isStructureChanged){
		++undoRedoIndex;
		undoRedoArray[undoRedoIndex] = getDuplicateMatrix(tableMatrix);
	}
	else{
		isStructureChanged = true;
	}
	if(!isFromUndoRedo ){
		undoRedoArray.splice(undoRedoIndex+1, undoRedoArray.length-undoRedoIndex);
	}
	else {
		isFromUndoRedo = false;
	}
	if(undoRedoIndex > 0){
		document.getElementById('undoDisable').className = 'hide';
		document.getElementById('undoEnable').className = '';
	}
	else {
		document.getElementById('undoEnable').className = 'hide';
		document.getElementById('undoDisable').className = '';
	}
	if(undoRedoArray.length -1 > undoRedoIndex){
		document.getElementById('redoDisable').className = 'hide';
		document.getElementById('redoEnable').className = '';
	}
	else {
		document.getElementById('redoEnable').className = 'hide';
		document.getElementById('redoDisable').className = '';
	}

	if(_columnCount > 3){
		width = '100%'
	}
	var outerTable = "<TABLE width='"+width+"' class='customLayout'>";
	var size = viewNames.length;
	var previousRowIndex = -1;
//	outerTable = outerTable.concat("<TR><TD></TD>")
//	for(var i=0; i < _columnCount; i++){
//		outerTable = outerTable.concat("<TD align='center' class='horIndicator'>"+(i+1)+"</TD>")
//	}
//	outerTable = outerTable.concat("</TR>")
	for(var i = 0; i < size; i++){
		var viewProps = viewObjects[viewNames[i]];
		var rowIndex = viewProps["ROWINDEX"];
		var rowSpan = viewProps["ROWSPAN"];
		var columnIndex = viewProps["COLUMNINDEX"];
		var columnSpan = viewProps["COLSPAN"];
		var description = viewProps["DESCRIPTION"];
		var isDummy = viewProps["ISDUMMY"];
		if(previousRowIndex != rowIndex){
			if(rowIndex > 1){
				outerTable = outerTable.concat("</TR>");
			}
			previousRowIndex = rowIndex;
			outerTable = outerTable.concat("<TR>");
	//		outerTable = outerTable.concat("<TD class='verIndicator' align='center'>&nbsp;"+(rowIndex+1)+"&nbsp;</TD>")
		}
		var isLastDragged = false;
		if(draggedViewName != null && draggedViewName == viewProps["VIEWID"]){
			isLastDragged = true;
		}

		outerTable = outerTable.concat(executeElementCall("getTableCellConstruct",new Array(columnSpan, rowSpan, columnSpan, rowSpan, isLastDragged)));
		if(currentViewInEdit == viewNames[i]){
		}
		outerTable = outerTable.concat(executeElementCall("createInnerTable", new Array(viewProps, i)));
		outerTable = outerTable.concat("</TD>");
	}
//	outerTable = outerTable.concat("</TR><TR><TD class='verIndicator'></TD><TD height='10px'colspan='"+_columnCount+"' onMouseOver='executeElementCall(\"enableViewSelection\",this)' viewId='-1' id='_evl' class='emptyCell' align='center'>(Drag & Drop here to add components to the last)</TD>");
	outerTable = outerTable.concat("</TR></TABLE>");
	element = document.getElementById("layoutDAC");
	if(element != null){
		element.innerHTML = outerTable;
	}
	executeElementCall("constructRemovedViews");
	draggedViewName = null;
}

/**
 *
 */
function addMethod(buffer, method, x, y, viewName){
	buffer = buffer.concat(method);
	buffer = buffer.concat("(");
	buffer = buffer.concat(x);
	buffer = buffer.concat(", ");
	buffer = buffer.concat(y);
	if(viewName != null){
	buffer = buffer.concat(", ");
	buffer = buffer.concat("'");
	buffer = buffer.concat(viewName);
	buffer = buffer.concat("'");
	}
	buffer = buffer.concat(")");
	return buffer;
}

function getTopButtons(viewProps){
	var y = viewProps["Y"];
	var x = viewProps["X"];
	var viewName = viewProps["VIEWID"];
	var colspan = viewProps["COLSPAN"];
	var rowspan = viewProps["ROWSPAN"];
	var isDummy = viewProps["ISDUMMY"];
	var toolbarTable = "";
//	if(x - (rowspan - 1) > 0){
		toolbarTable = toolbarTable.concat("<BUTTON class = 'expandTop' ");
		toolbarTable = toolbarTable.concat("onClick= 'return ");
		toolbarTable = addMethod(toolbarTable, 'increaseHeightTop', x, y);
		toolbarTable = toolbarTable.concat("' >&nbsp;</BUTTON>");
//	}
//	else {
//		toolbarTable = toolbarTable.concat("<BUTTON class = 'decrementHeightDisabled' >&nbsp;</BUTTON>");
//	}	
	if(rowspan > 1){
			toolbarTable = toolbarTable.concat("<BUTTON class = 'expandBottom' ");
			toolbarTable = toolbarTable.concat("onClick= 'return ");
			toolbarTable = addMethod(toolbarTable, 'decreaseHeightTop', x, y);
			toolbarTable = toolbarTable.concat("' >&nbsp;</BUTTON>");
	}
	else {
			toolbarTable = toolbarTable.concat("<BUTTON class = 'expandBottomDis' >&nbsp;</BUTTON>");
	}

	return toolbarTable;	
}

function getBottomButtons(viewProps){
	var y = viewProps["Y"];
	var x = viewProps["X"];
	var viewName = viewProps["VIEWID"];
	var colspan = viewProps["COLSPAN"];
	var rowspan = viewProps["ROWSPAN"];
	var isDummy = viewProps["ISDUMMY"];
	var toolbarTable = "";
//	if(x != (rowCount -1)){
		toolbarTable = toolbarTable.concat("<BUTTON class = 'expandBottom' ");
		toolbarTable = toolbarTable.concat("onClick= 'return ");
		toolbarTable = addMethod(toolbarTable, 'increaseHeightBottom', x, y);
		toolbarTable = toolbarTable.concat("' >&nbsp;</BUTTON>");
//	}
//	else {
//		toolbarTable = toolbarTable.concat("<BUTTON class = 'incrementHeightDisabled' >&nbsp;</BUTTON>");
//	}
	if(rowspan > 1){
		toolbarTable = toolbarTable.concat("<BUTTON class = 'expandTop' ");
		toolbarTable = toolbarTable.concat("onClick= 'return ");
		toolbarTable = addMethod(toolbarTable, 'decreaseHeightBottom', x, y);
		toolbarTable = toolbarTable.concat("' >&nbsp;</BUTTON>");
	}
	else {
		toolbarTable = toolbarTable.concat("<BUTTON class = 'expandTopDis' >&nbsp;</BUTTON>");
	}
	return toolbarTable;
}

function getLeftButtons(viewProps){
	var y = viewProps["Y"];
	var x = viewProps["X"];
	var viewName = viewProps["VIEWID"];
	var colspan = viewProps["COLSPAN"];
	var rowspan = viewProps["ROWSPAN"];
	var isDummy = viewProps["ISDUMMY"];
	var toolbarTable = "";
//	if(y - (colspan-1) > 0){
		toolbarTable = toolbarTable.concat("<BUTTON class = 'expandLeft' ");
		toolbarTable = toolbarTable.concat("onClick= 'return ");
		toolbarTable = addMethod(toolbarTable, 'increaseWidthLeft', x, y);
		toolbarTable = toolbarTable.concat("' >&nbsp;</BUTTON>");
//	}
//	else {
//		toolbarTable = toolbarTable.concat("<BUTTON class = 'decrementWidthDisabled'>&nbsp;</BUTTON>");
//	}
		if(colspan > 1){
		toolbarTable = toolbarTable.concat("<BUTTON class = 'expandRight' ");
		toolbarTable = toolbarTable.concat("onClick= 'return ");
		toolbarTable = addMethod(toolbarTable, 'decreaseWidthRight', x, y);
		toolbarTable = toolbarTable.concat("' >&nbsp;</BUTTON>");

	}
	else {
		toolbarTable = toolbarTable.concat("<BUTTON class = 'expandRightDis' >&nbsp;</BUTTON>");
	}

	return toolbarTable;
}

function getRightButtons(viewProps){
	var y = viewProps["Y"];
	var x = viewProps["X"];
	var viewName = viewProps["VIEWID"];
	var colspan = viewProps["COLSPAN"];
	var rowspan = viewProps["ROWSPAN"];
	var isDummy = viewProps["ISDUMMY"];
	var toolbarTable = "";
/*	if(y != (_columnCount - 1)){*/
		toolbarTable = toolbarTable.concat("<BUTTON class = 'expandRight' ");
		toolbarTable = toolbarTable.concat("onClick= 'return ");
		toolbarTable = addMethod(toolbarTable, 'increaseWidthRight', x, y);
		toolbarTable = toolbarTable.concat("' >&nbsp;</BUTTON>");
/*	}
	else {
		toolbarTable = toolbarTable.concat("<BUTTON class = 'incrementWidthDisabled'>&nbsp;</BUTTON>");
	}*/
	if(colspan > 1){
		toolbarTable = toolbarTable.concat("<BUTTON class = 'expandLeft' ");
		toolbarTable = toolbarTable.concat("onClick= 'return ");
		toolbarTable = addMethod(toolbarTable, 'decreaseWidthLeft', x, y);
		toolbarTable = toolbarTable.concat("' >&nbsp;</BUTTON>");
	}
	else {
			toolbarTable = toolbarTable.concat("<BUTTON class = 'expandLeftDis' >&nbsp;</BUTTON>");
	}

	return toolbarTable;
}

